<input
    id="{{ field.html_id }}"
    type="checkbox"
    name="{{ field.html_name }}"
    class="rounded"
    {% if field.value() %}checked{% endif %}
    {% if field.field.required %}required{% endif %}
    >
